#!/usr/bin/env python

import os
import json
import subprocess
import sys
import time


def juju_info(msg):
    subprocess.check_call(['juju-log', '-l', 'INFO', msg])
    pass


def _config():
    output = subprocess.check_output(['config-get', '--format=json'])
    return json.loads(output)


def _relation_ids(relation_types=('db',)):
    # accept strings or iterators
    if isinstance(relation_types, basestring):
        reltypes = [relation_types, ]
    else:
        reltypes = relation_types
    relids = []
    for reltype in reltypes:
        relid_cmd_line = ['relation-ids', '--format=json', reltype]
        json_relids = subprocess.check_output(relid_cmd_line).strip()
        if json_relids:
            relids.extend(json.loads(json_relids))
    return relids


def _relation_set(keyvalues, relation_id=None):
    args = ['relation-set']
    if relation_id:
        args.extend(['-r', relation_id])
    args.extend(["{}='{}'".format(k, v or '') for k, v in keyvalues.items()])
    subprocess.check_call(args)


def _service_dir(config):
    unit = os.environ['JUJU_UNIT_NAME'].split('/')[0]
    for x in (':', '-', '/', '"', "'"):
        unit = unit.replace(x, '_')
    return os.path.join(config['install_root'], unit)


def install(config):
    pkgs = [x for x in config['packages'].split(' ') if x]
    pkgs.append('bzr')
    pkgs.append('python-pip')  # required for restish?

    juju_info('installing apt packages...')
    subprocess.check_call(['apt-get', 'install', '-y', '-q'] + pkgs)

    juju_info('installing restish...')
    restish = 'restish==%s' % config['restish_version']
    subprocess.check_call(['pip', 'install', restish])

    juju_info('grabbing service from bzr...')
    args = ['bzr', 'branch']
    rev = config.get('revno', '')
    if rev:
        args.extend(['-r', rev])
    args.append(config['branch'])
    args.append(_service_dir())
    subprocess.check_call(args)


def start(config):
    service = 'restish'
    conf = '/etc/init/%s.sh' % service
    if os.path.exists(conf):
        try:
            subprocess.check_call(['service', 'restart', service])
        except subprocess.CalledProcessError:
            # if it wasn't running, restart fails, so just try to start
            subprocess.check_call(['service', 'start', service])


def stop(config):
    service = 'restish'
    conf = '/etc/init/%s.sh' % service
    if os.path.exists(conf):
        subprocess.check_call(['service', 'stop', service])


def config_changed(config):
    # Trigger WSGI reloading
    for relid in _relation_ids('wsgi'):
        _relation_set({'wsgi_timestamp': time.time()}, relation_id=relid)


def website_relation_joined(config):
    host = subprocess.check_output(['unit-get', 'private-address']).strip()
    _relation_set({'port': config["port"], 'hostname': host})


def website_relation_changed(config):
    return website_relation_joined(config)


def website_relation_broken(config):
    pass


def wsgi_relation_joined(config):
    for var in config:
        if var.startswith('wsgi_') or var in ['listen_ip', 'port']:
            _relation_set({var: config[var]})

    path = config.get('python_path')
    if path:
        sdir = _service_dir(config)
        full_path = []
        for p in path.split(':'):
            if p[0] != '/':
                p = os.path.join(sdir, p)
            full_path.append(p)
        _relation_set({'python_path': ':'.join(full_path)})

    port = config.get('port')
    if port:
        subprocess.call(['open-port', "%s/TCP" % port])


def wsgi_relation_changed(config):
    return wsgi_relation_joined(config)


def wsgi_relation_broken(config):
    port = config.get('port')
    if port:
        subprocess.call(['close-port', "%s/TCP" % port])


def main():
    hook = os.path.basename(sys.argv[0])
    juju_info("Running hook: %s" % hook)

    hook_py = hook.replace('-', '_')
    funcs = globals()
    if hook_py not in funcs:
        print("Unknown hook: %s" % hook)
        return 1

    config = _config()
    try:
        return funcs[hook_py](config)
    except subprocess.CalledProcessError as e:
        juju_info('Error running: %s: %s' % (e.cmd, e.output))
        return e.returncode


if __name__ == '__main__':
    exit(main())
